#!/usr/bin/env python
"""
Little script to make an executable into a UNIX daemon

Uses prescription from UNIX programming FAQ
http://www.erlenstar.demon.co.uk/unix/
"""

import sys, os

def usage():
    sys.stdout.write("Usage: rgma-daemonize.py [-h|--help|-v|--version] | <command>\n")

def version():
    sys.stdout.write("rgma-daemonize.py version 5.0.7\n")

def help():
    usage()
    sys.stdout.write("\nUtility to turn an executable into a UNIX daemon.\n")
    sys.stdout.write("Outputs the PID of the forked process\n")
    
if __name__ == "__main__":

    if len(sys.argv) < 2:
        usage()
        sys.exit(1)

    if sys.argv[1] in ("--help", "-h"):
        help()
        sys.exit(0)
    elif sys.argv[1] in ("--version", "-v"):
        version()
        sys.exit(0)

    # Fork and let parent exit
    try: 
        pid = os.fork() 
        if pid > 0:
            sys.exit(0) 
    except OSError, e: 
        sys.stderr.write("Error on first fork: %d (%s)\n" % (e.errno, e.strerror))
        sys.exit(1)

    # Become a process and session group leader
    os.setsid() 

    # Fork again so session group leader can exit (i.e. so daemon can have
    # no session group leader)
    try: 
        pid = os.fork() 
        if pid > 0:
            # Display daemon PID
            print pid 
            sys.exit(0) 
    except OSError, e: 
        sys.stderr.write("Error on second fork: %d (%s)\n" % (e.errno, e.strerror))
        sys.exit(1) 

    # Ensure we don't keep any directory in use
    os.chdir("/")

    # Set the default permissions for newly created files to be user-writable only
    os.umask(022)
    
    # Close stdin/stdout/stderr because we don't know what they're connected
    # to. Note that this is a bit funny in Python - you need to close the python
    # objects as well as the underlying OS file descriptors
    sys.stdin.close()
    sys.stdout.close()
    sys.stderr.close()
    os.close(0)
    os.close(1)
    os.close(2)

    # Create new stdin/stdout/stderr (going nowhere)
    sys.stdin = open("/dev/null")
    sys.stdout = open("/dev/null")
    sys.stderr = open("/dev/null")
    
    # start the daemon using the specified command
    os.execvp(sys.argv[1], sys.argv[1:])
