#include "rgma/ConsumerFactory.h"
#include "impl/ConsumerFactoryImpl.h"
#include "rgma/Consumer.h"
#include "rgma/QueryProperties.h"
#include "rgma/ResultSet.h"
#include "rgma/TimeInterval.h"
#include "rgma/Units.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"

#include <string>
#include <iostream>
#include <unistd.h>

using glite::rgma::ConsumerFactory;
using rgma::impl::ConsumerFactoryImpl;
using glite::rgma::Consumer;
using glite::rgma::QueryProperties;
using glite::rgma::ResultSet;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::RemoteException;
using glite::rgma::UnknownResourceException;
using glite::rgma::RGMAException;

int main(int argc, char* argsv[]) {

  ConsumerFactory* factory = new ConsumerFactoryImpl();

  try {

    TimeInterval terminationInterval(60, Units::MINUTES);
    Consumer* c = factory->createConsumer(terminationInterval, 
                                          "SELECT * FROM userTable", 
                                          QueryProperties::CONTINUOUS);
 
    c->start(TimeInterval(300, Units::SECONDS));

    bool endOfResults = false;
    while (!endOfResults) {
      sleep(5);
      ResultSet resultSet;
      c->pop(resultSet, 2000);
      std::cout << resultSet << std::endl;
      endOfResults = resultSet.endOfResults();
    }
  
    c->close();

  }
  
  catch (RemoteException e) {
    std::cerr << "Failed to contact Consumer service."<< std::endl;
    std::cout << e.getMessage() << std::endl;
  }
  catch (UnknownResourceException ure) {
    std::cerr << "Failed to contact Consumer service."<< std::endl;
    std::cerr << ure.getMessage() << std::endl;
  }
  catch (RGMAException rgmae) {
    std::cerr << "R-GMA application error in Consumer."<< std::endl;
    std::cout << rgmae.getMessage() << std::endl;
  }

  delete factory;

}
