#include "impl/ProducerFactoryImpl.h"
#include "rgma/RGMAException.h"
#include "rgma/RemoteException.h"
#include "rgma/ExceptionBase.h"
#include "rgma/QueryProperties.h"
#include "rgma/ProducerProperties.h"
#include "rgma/TimeInterval.h"
#include "rgma/Storage.h"
#include "rgma/Units.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/PrimaryProducer.h"
#include "rgma/ProducerFactory.h"

#include <string>
#include <iostream>
#include <stdlib.h>

using glite::rgma::ExceptionBase;
using glite::rgma::PrimaryProducer;
using glite::rgma::ProducerProperties;
using glite::rgma::ProducerFactory;
using rgma::impl::ProducerFactoryImpl;
using glite::rgma::QueryProperties;
using glite::rgma::RGMAException;
using glite::rgma::Storage;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::UnknownResourceException;
using glite::rgma::RemoteException;


int main(int argc, char* argv[]) { 
  if (argc != 2)  {
    std::cout<< "Usage: PrimaryProducerExample <userId>\n"<< std::endl;
    exit(1);
  }


  ProducerFactory* factory = new ProducerFactoryImpl();


  try {

    TimeInterval terminationInterval(60, Units::MINUTES);
    ProducerProperties props(Storage::MEMORY);
    PrimaryProducer* pp = factory->createPrimaryProducer(terminationInterval,
                                                         props);

    std::string userTable("userTable");
    std::string args(argv[1]);
    std::string predicate("WHERE userId = '" + args + "'");
    TimeInterval historyRP(60, Units::MINUTES);
    TimeInterval latestRP(60, Units::MINUTES);
    pp->declareTable(userTable,  predicate, historyRP, latestRP);

    std::string insert;
    insert.append("INSERT INTO userTable (userId, aString, aReal, anInt)");
    insert.append(" VALUES ('");
    insert.append(args);
    insert.append("', 'C++ producer', 3.1415926, 42)"); 
    pp->insert(insert); 

    pp->close();
  }

  catch (RemoteException e) {
    std::cerr << "Failed to contact PrimaryProducer service."<< std::endl;
    std::cout << e.getMessage() << std::endl;
  }
  catch (UnknownResourceException ure) {
    std::cerr << "Failed to contact PrimaryProducer service."<< std::endl;
    std::cerr << ure.getMessage() << std::endl;
  }
  catch (RGMAException rgmae) {
    std::cerr << "R-GMA application error in PrimaryProducer."<< std::endl;
    std::cout << rgmae.getMessage() << std::endl;
  }

  delete factory;
    
}
