#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "rgma.h"

int main(int argc, char *argv[]) {

  RGMAResource  *consumer;
  RGMAResultSet *rs;
  const RGMAException *exception;
  int terminationIntervalSec = 60 * 60;
  int timeoutSec = 365 * 24 * 60 * 60;
  int oldDataSec = 30;
  const char *select = "SELECT * FROM userTable";

  int consumerCreated = 0;
  int consumerStarted = 0;
  int endOfResults = 0;
 
  while(1) {

    while (!consumerCreated) {

      if (RGMA_createConsumer(terminationIntervalSec, select, 
                              RGMAQueryType_CONTINUOUS | RGMAQueryType_INTERVAL, 
                              oldDataSec, &consumer) != 0) {
        exception = RGMA_getException(consumer);
        if (exception->errorType == RGMA_REMOTEEXCEPTION) {
          fprintf(stderr, "Failed to contact R-GMA server: %s - will retry in 60s\n", 
                  exception->errorMessage);
          RGMA_freeResource(consumer);
          sleep(60);
        }
        else {
          fprintf(stderr, "Unexpected R-GMA error: %s - exiting\n", 
                  exception->errorMessage);
          RGMA_freeResource(consumer);
          exit(1);
        }
      }
      else {
        consumerCreated=1;
      }
    }

    while(!consumerStarted) {
      
      if (RGMAConsumer_start(consumer, timeoutSec) != 0) {
        exception = RGMA_getException(consumer);
        if (exception->errorType == RGMA_REMOTEEXCEPTION) {
          fprintf(stderr, "Failed to contact R-GMA server: %s - will retry in 60s\n", 
                  exception->errorMessage);
          sleep(60);
        }
        else if (exception->errorType == RGMA_UNKNOWNRESOURCEEXCEPTION) {
          fprintf(stderr, "Consumer has died - will try to create a new one\n");
          RGMA_freeResource(consumer);
          consumerCreated = 0;
          consumerStarted = 0;
          endOfResults = 0;
          break;
        }
        else {
          fprintf(stderr, "Unexpected R-GMA error: %s - exiting\n", 
                  exception->errorMessage);
          RGMA_close(consumer);
          RGMA_freeResource(consumer);
          exit(1);
        }
      }
      else {
        consumerStarted = 1;
      }
    }

    if (consumerStarted) {
      while (!endOfResults) {
        sleep(5);
        rs = RGMAConsumer_pop(consumer, 2000);
        if (rs == NULL) {
          exception = RGMA_getException(consumer);
          if (exception->errorType == RGMA_REMOTEEXCEPTION) {
            fprintf(stderr, "Failed to contact R-GMA server: %s - will retry in 60s\n", 
                    exception->errorMessage);
            sleep(60);
          }
          else if (exception->errorType == RGMA_UNKNOWNRESOURCEEXCEPTION) {
            fprintf(stderr, "Consumer has died - will try to create a new one\n");
            RGMA_freeResource(consumer);
            consumerCreated = 0;
            consumerStarted = 0;
            endOfResults = 0;
            break;
          }
          else {
            fprintf(stderr, "Unexpected R-GMA error: %s - exiting\n", 
                    exception->errorMessage);
            RGMA_close(consumer);
            RGMA_freeResource(consumer);
            exit(1);
          }
        } 
        else {
          if (rs->numRows > 0) {
            RGMA_printResultSet(stdout, rs);
          }
          endOfResults = rs->endOfResults;
          RGMA_freeResultSet(rs);
        }
      }
    }

    if (endOfResults) {
      RGMA_close(consumer);
      RGMA_freeResource(consumer);
      consumerCreated = 0;
      consumerStarted = 0;
      endOfResults = 0;
    } 
  }
}
