#include "rgma/ConsumerFactory.h"
#include "impl/ConsumerFactoryImpl.h"
#include "rgma/Consumer.h"
#include "rgma/QueryProperties.h"
#include "rgma/ResultSet.h"
#include "rgma/TimeInterval.h"
#include "rgma/Units.h"
#include "rgma/RemoteException.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/RGMAException.h"

#include <string>
#include <iostream>
#include <unistd.h>

using glite::rgma::ConsumerFactory;
using rgma::impl::ConsumerFactoryImpl;
using glite::rgma::Consumer;
using glite::rgma::QueryProperties;
using glite::rgma::ResultSet;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::RemoteException;
using glite::rgma::UnknownResourceException;
using glite::rgma::RGMAException;

int main(int argc, char* argsv[]) {
  
  Consumer *consumer = NULL;
  ConsumerFactory* factory = new ConsumerFactoryImpl();
  TimeInterval terminationInterval(60, Units::MINUTES);
  TimeInterval oldData(30, Units::SECONDS);
  TimeInterval timeout(3600, Units::SECONDS);
  std::string select("SELECT * FROM userTable");
  
  bool consumerCreated = false;
  bool consumerStarted = false;
  bool endOfResults = false;
  
  while(true) {
    try {
      while (!consumerCreated) {
        consumer = factory->createConsumer(terminationInterval, select, 
                                           QueryProperties::getContinuous(oldData));
        consumerCreated = true;
      }
      
      while (!consumerStarted) {
        consumer->start(timeout);
        consumerStarted = true;
      }
      
      while(!endOfResults) {
        sleep(5);
        ResultSet resultSet;
        consumer->pop(resultSet, 2000);
        if(resultSet.begin() != resultSet.end()) {
          std::cout << resultSet << std::endl;
        }
        std::string warning = resultSet.getWarning().getMessage();
        if (warning.length() > 0) {
          std::cout << "WARNING: " << resultSet.getWarning().getMessage() << std::endl;
        }
        endOfResults = resultSet.endOfResults();
      }
      
      if (consumer != NULL) {
        try {consumer->close();} catch (...) {}
        delete consumer;
        consumer = NULL;
      }
      consumerCreated = false;
      consumerStarted = false;
      endOfResults = false;
    }
    catch (UnknownResourceException ure) {
      std::cerr << "Consumer has died - will try to create a new one"<< std::endl;
      if (consumer != NULL) delete consumer;
      consumer = NULL;
      consumerCreated = false;
      consumerStarted = false;
      endOfResults = false;
    }  
    catch (RemoteException e) {
      std::cerr << "Failed to contact R-GMA server: " << e.getMessage() 
                << " - will retry in 60s" << std::endl;
      sleep(60);
    }
    catch (RGMAException rgmae) {
      std::cerr << "Unexpected R-GMA error - exiting" << std::endl;
      std::cerr << rgmae.getMessage() << std::endl;
      if (consumer != NULL) {
        try {consumer->close();} catch (...) {}
        delete consumer;
        consumer = NULL;
      }
      delete factory;
      exit(1);
    } 
  }
}
