import org.glite.rgma.Consumer;
import org.glite.rgma.ConsumerFactory;
import org.glite.rgma.QueryProperties;
import org.glite.rgma.RGMAException;
import org.glite.rgma.RemoteException;
import org.glite.rgma.ResultSet;
import org.glite.rgma.TimeInterval;
import org.glite.rgma.Units;
import org.glite.rgma.UnknownResourceException;
import org.glite.rgma.stubs.ConsumerFactoryStub;

public class ResilientConsumer {
    public static void main(String[] args) {

        Consumer consumer = null;
        ConsumerFactory factory = new ConsumerFactoryStub();
        TimeInterval terminationInterval = new TimeInterval(60, Units.MINUTES);
        TimeInterval oldData = new TimeInterval(30, Units.SECONDS);
        String select = "SELECT * FROM userTable";

        boolean consumerCreated = false;
        boolean consumerStarted = false;
        boolean endOfResults = false;

        while (true) {
            try {
                while (!consumerCreated) {
                    QueryProperties queryProps = QueryProperties.getContinuous(oldData);
                    consumer = factory.createConsumer(terminationInterval, select,
                                                      queryProps);
                    consumerCreated = true;
                }

                while (!consumerStarted) {
                    consumer.start(new TimeInterval(365, Units.DAYS));
                    consumerStarted = true;
                }

                while (!endOfResults) {
                    try {Thread.sleep(5 * 1000);} catch (InterruptedException ie) {}
                    ResultSet rs = consumer.pop(2000);
                    if (rs.next()) {
                        System.out.println(rs);
                    }
                    if (rs.getWarning() != null) {
                        System.out.println("WARNING: " + rs.getWarning().getMessage());
                    }
                    endOfResults = rs.endOfResults();
                }

                if (consumer != null) try {consumer.close();} catch (Exception ex) {}

                consumerCreated = false;
                consumerStarted = false;
                endOfResults = false;
            } catch (UnknownResourceException e) {
                System.err.println("Consumer has died - will try to create a new one");
                consumerCreated = false;
                consumerStarted = false;
                endOfResults = false;
            } catch (RemoteException e) {
                System.err.println("Failed to contact R-GMA server: " + e.getMessage() +
                                   " - will retry in 60s\n");
                try {Thread.sleep(60 * 1000);} catch (InterruptedException ie) {}
            } catch (RGMAException e) {
                System.err.println("Unexpected R-GMA error - exiting");
                if (consumer != null) try {consumer.close();} catch (Exception ex) {}
                e.printStackTrace(System.err);
                System.exit(1);
            }
        }
    }
}
