#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "rgma.h"

#define MAX_SQL 255

int main(int argc, char *argv[]) 
{
  RGMAResource  *producer;
  int           historyRPSec, latestRPSec, terminationIntervalSec;
  char          insert[MAX_SQL], predicate[MAX_SQL];
  int           data;
  int           producerCreated, tableDeclared;
  const RGMAException *exception;

  if (argc != 2) {
    fprintf(stderr, "Usage: %s <userId>\n", argv[0]);
    exit(1);
  }

  producer = NULL;
  data = 0;
  sprintf(predicate, "WHERE userId = '%s'", argv[1]);
  terminationIntervalSec = 60*60;
  historyRPSec = 50*60;
  latestRPSec = 25*60;

  producerCreated = 0;
  tableDeclared = 0;

  while (1) {

    while(!producerCreated) {
      if (RGMA_createPrimaryProducer(3600, 0, 
                                     RGMAStorageType_MEMORY, NULL, 
                                     &producer) != 0) {
        exception = RGMA_getException(producer);
        if (exception->errorType == RGMA_REMOTEEXCEPTION) {
          fprintf(stderr, "Failed to contact R-GMA server: %s - will retry in 60s\n", 
                  exception->errorMessage);
          RGMA_freeResource(producer);
          sleep(60);
        }
        else {
          fprintf(stderr, "Unexpected R-GMA error: %s - exiting\n",
                  exception->errorMessage);
          exit(1);
        }
      }
      else {
        producerCreated = 1;
      }
    }

    while (!tableDeclared) {
      if (RGMAPrimaryProducer_declareTable(producer,
                                           "userTable", predicate,          
                                           historyRPSec,
                                           latestRPSec) != 0) {
        exception = RGMA_getException(producer);
        if (exception->errorType == RGMA_REMOTEEXCEPTION) {
          fprintf(stderr, "Failed to contact R-GMA server: %s - will retry in 60s\n",
                  exception->errorMessage);
          sleep(60);
        }
        else if (exception->errorType == RGMA_UNKNOWNRESOURCEEXCEPTION) {
          fprintf(stderr, "Producer has died - will try to create a new one\n");
          RGMA_freeResource(producer);
          producerCreated = 0;
          tableDeclared = 0;
          break;
        }
        else {
          fprintf(stderr, "Unexpected R-GMA error: - exiting\n");
          fprintf(stderr, "%s\n", exception->errorMessage);
          if (producer != NULL) RGMA_close(producer);
          exit(1);
        }
      }
      else {
        tableDeclared = 1;
      }
    }
    
    if(tableDeclared) {    
      while (1) {
        sprintf(insert, "INSERT INTO userTable (userId, aString, aReal, anInt) \
                         VALUES ('%s', '', 0.0 , %d)", 
                argv[1], data);
        
        if (RGMAPrimaryProducer_insert(producer, insert, 0) != 0) {
          exception = RGMA_getException(producer);
          if (exception->errorType == RGMA_REMOTEEXCEPTION) {
            fprintf(stderr, "Failed to contact R-GMA server: %s - will retry in 60s\n",
                    exception->errorMessage);
            sleep(60);
          }
          else if (exception->errorType == RGMA_UNKNOWNRESOURCEEXCEPTION) {
            fprintf(stderr, "Producer has died - will try to create a new one\n");
            RGMA_freeResource(producer);
            producerCreated = 0;
            tableDeclared = 0;
            break;
          }
          else if (exception->errorNumber == RGMA_BUFFERFULLEXCEPTION) {
            fprintf(stderr, "Producer buffer is full - will retry insert in 60s\n");
            sleep(60);
          }
          else if (exception->errorNumber == RGMA_USEREXCEPTION) {
            fprintf(stderr, "WARNING: Invalid tuple %s (%s)\n", insert,
                    exception->errorMessage);
          }
          else {
            fprintf(stderr, "Unexpected R-GMA error: - exiting\n");
            fprintf(stderr, "%s\n", exception->errorMessage);
            if (producer != NULL) RGMA_close(producer);
            exit(1);
          }
        }
        else {
          printf("%s\n", insert);
          data++;
          sleep(30);
        }
      }
    }
  }
}
  


