#include "impl/ProducerFactoryImpl.h"
#include "rgma/RGMABufferFullException.h"
#include "rgma/RGMAException.h"
#include "rgma/RGMAUserException.h"
#include "rgma/RemoteException.h"
#include "rgma/ExceptionBase.h"
#include "rgma/QueryProperties.h"
#include "rgma/ProducerProperties.h"
#include "rgma/TimeInterval.h"
#include "rgma/Storage.h"
#include "rgma/Units.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/PrimaryProducer.h"
#include "rgma/ProducerFactory.h"

#include <string>
#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <unistd.h>

using glite::rgma::ExceptionBase;
using glite::rgma::PrimaryProducer;
using glite::rgma::ProducerProperties;
using glite::rgma::ProducerFactory;
using rgma::impl::ProducerFactoryImpl;
using glite::rgma::QueryProperties;
using glite::rgma::RGMABufferFullException;
using glite::rgma::RGMAException;
using glite::rgma::RGMAUserException;
using glite::rgma::Storage;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::UnknownResourceException;
using glite::rgma::RemoteException;

int main(int argc, char* argv[]) { 

  if (argc != 2)  {
    std::cerr << "Usage: " << argv[0] << " <userId>" << std::endl;
    exit(1);
  }

  std::string userId(argv[1]);
  ProducerFactory* factory = new ProducerFactoryImpl();
  PrimaryProducer* producer = NULL;
  int data = 0;
  std::string predicate("WHERE userId = '" + userId + "'");
  ProducerProperties props(Storage::MEMORY);
  TimeInterval terminationInterval(60, Units::MINUTES);
  TimeInterval historyRP(50, Units::MINUTES);
  TimeInterval latestRP(25, Units::MINUTES);
  std::stringstream insert;

  bool producerCreated = false;
  bool tableDeclared = false;
  
  while(true) {

    try {
        
        while( !producerCreated ) {
            producer = factory->createPrimaryProducer(terminationInterval, props);
            producerCreated = true;
        }
        
        while(!tableDeclared) {
            producer->declareTable("userTable",  predicate, historyRP, latestRP);
            tableDeclared = true;
        }
        
        while(true) {
          insert.str("");
          insert << "INSERT INTO userTable (userId, aString, aReal, anInt) "
                 << "VALUES('" << userId << "', '', 0.0, " << data << ")";
          producer->insert(insert.str());
          std::cout << insert.str() << std::endl;
          data++;
          sleep(30);
        }
    }

    catch (UnknownResourceException ure) {
        std::cerr << "Producer has died - will try to create a new one" << std::endl;
        if (producer != NULL) delete(producer);
        producer = NULL;
        tableDeclared = false;
        producerCreated = false;
    }
    catch (RemoteException e) {
      std::cerr << "Failed to contact R-GMA server: " << e.getMessage() << 
        " - will retry in 60s" << std::endl;
      sleep(60);
    }
    catch (RGMABufferFullException bfe) {
      std::cerr << "Producer buffer is full - will retry insert in 60s" << std::endl;
      sleep(60);
    }
    catch (RGMAUserException ue) {
      std::cerr << "WARNING: Invalid tuple: " << insert << "(" << ue.getMessage() << 
        ")" << std::endl;
    }
    catch (RGMAException rgmae) {
      std::cerr << "Unexpected R-GMA error - exiting" << std::endl;
      std::cerr << rgmae.getMessage() << std::endl;
      if (producer != NULL) {
        try {producer->close();} catch (...) {}
        delete producer;
        producer = NULL;
      }
      delete factory;
      exit(1);
    }
  }
}
