import org.glite.rgma.PrimaryProducer;
import org.glite.rgma.ProducerFactory;
import org.glite.rgma.ProducerProperties;
import org.glite.rgma.RGMABufferFullException;
import org.glite.rgma.RGMAUserException;
import org.glite.rgma.RGMAException;
import org.glite.rgma.RemoteException;
import org.glite.rgma.Storage;
import org.glite.rgma.TimeInterval;
import org.glite.rgma.Units;
import org.glite.rgma.UnknownResourceException;
import org.glite.rgma.stubs.ProducerFactoryStub;

public class ResilientPrimaryProducer {
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: java ResilientPrimaryProducer <userId>");
            System.exit(1);
        }

        ProducerFactory factory = new ProducerFactoryStub();
        PrimaryProducer producer = null;
        int data = 0;
        String predicate = "WHERE userId = '" + args[0] + "'";
        ProducerProperties props = new ProducerProperties(Storage.MEMORY, 0);
        TimeInterval terminationInterval = new TimeInterval(60, Units.MINUTES);
        TimeInterval historyRP = new TimeInterval(50, Units.MINUTES);
        TimeInterval latestRP = new TimeInterval(25, Units.MINUTES);
        String insert = null;

        boolean producerCreated = false;
        boolean tableDeclared = false;

        while(true) {
            try {
                while(!producerCreated){
                    producer = factory.createPrimaryProducer(terminationInterval, props);
                    producerCreated = true;
                }

                while(!tableDeclared){
                    producer.declareTable("userTable", predicate, historyRP, latestRP);
                    tableDeclared = true;
                }

                while(true) {
                    insert = "INSERT INTO userTable " +
                        "(userId, aString, aReal, anInt) VALUES " +
                        "('" + args[0] + "', '', 0.0, " + data + ")";
                    producer.insert(insert);
                    System.out.println(insert);
                    data++;
                    try {Thread.sleep(30 * 1000);} catch(InterruptedException ie) {}
                }
            } catch (UnknownResourceException e) {
                System.err.println("Producer has died - will try to create a new one");
                producerCreated = false;
                tableDeclared = false;
            } catch (RemoteException e) {
                System.err.println("Failed to contact R-GMA server: " + e.getMessage() +
                                   " - will retry in 60s");
                try {Thread.sleep(60 * 1000);} catch (InterruptedException ie) {}
            } catch (RGMABufferFullException e) {
                System.err.println("Producer buffer is full - will retry insert in 60s");
                try {Thread.sleep(60 * 1000);} catch (InterruptedException ie) {}
            } catch (RGMAUserException e) {
                System.err.println("WARNING: Invalid tuple: " + insert + "(" +
                                   e.getMessage() + ")");
            } catch (RGMAException e) {
                System.err.println("Unexpected R-GMA error - exiting");
                e.printStackTrace(System.err);
                if (producer != null) try {producer.close();} catch (Exception ex) {}
                System.exit(1);
            }
        }
    }
}
