#include "impl/ProducerFactoryImpl.h"
#include "rgma/RGMAException.h"
#include "rgma/RemoteException.h"
#include "rgma/ExceptionBase.h"
#include "rgma/QueryProperties.h"
#include "rgma/ProducerProperties.h"
#include "rgma/TimeInterval.h"
#include "rgma/Storage.h"
#include "rgma/Units.h"
#include "rgma/UnknownResourceException.h"
#include "rgma/PrimaryProducer.h"
#include "rgma/ProducerFactory.h"

#include <string>
#include <iostream>
#include <stdlib.h>
#include <unistd.h>

using glite::rgma::ExceptionBase;
using glite::rgma::PrimaryProducer;
using glite::rgma::ProducerProperties;
using glite::rgma::ProducerFactory;
using rgma::impl::ProducerFactoryImpl;
using glite::rgma::QueryProperties;
using glite::rgma::RGMAException;
using glite::rgma::Storage;
using glite::rgma::TimeInterval;
using glite::rgma::Units;
using glite::rgma::UnknownResourceException;
using glite::rgma::RemoteException;

int main(int argc, char* argv[]) { 

  ProducerFactory* factory = new ProducerFactoryImpl();
  
  try {
  
    TimeInterval ti(60, Units::MINUTES);
    std::string slocation("jdbc:mysql://localhost:3306/somedatabase");
    std::string username = "fred";
    std::string password = "bloggs";
    Storage storage = Storage::getDatabase(slocation, username, password);
    ProducerProperties props(storage,ProducerProperties::LATEST);
    SecondaryProducer* sp = factory->createSecondaryProducer(ti, props);
            
    TimeInterval historyRP(2, Units::HOURS);
    sp->declareTable("userTable", "", historyRP);
         
    while(true) {
      sp->showSignOfLife();
      sleep(55*60);
    }
        
  }
  
  catch (RemoteException e) {
    std::cerr << "Failed to contact SecondaryProducer service."<< std::endl;
    std::cout << e.getMessage() << std::endl;
  }
  catch (UnknownResourceException ure) {
    std::cerr << "Failed to contact SecondaryProducer service."<< std::endl;
    std::cerr << ure.getMessage() << std::endl;
  }
  catch (RGMAException rgmae) {
    std::cerr << "R-GMA application error in SecondaryProducer."<< std::endl;
    std::cout << rgmae.getMessage() << std::endl;
  }

  delete factory;
    
}
