import org.glite.rgma.ProducerFactory;
import org.glite.rgma.ProducerProperties;
import org.glite.rgma.RGMAException;
import org.glite.rgma.RemoteException;
import org.glite.rgma.SecondaryProducer;
import org.glite.rgma.Storage;
import org.glite.rgma.TimeInterval;
import org.glite.rgma.Units;
import org.glite.rgma.UnknownResourceException;
import org.glite.rgma.stubs.ProducerFactoryStub;


public class SecondaryProducerExample {
  public static void main(String[] args) {
    try {
      ProducerFactory factory = new ProducerFactoryStub();

      String slocation = "jdbc:mysql://localhost:3306/somedatabase";
      Storage storage = Storage.getDatabase(slocation, "fred", "bloggs");
      ProducerProperties props = new ProducerProperties(storage,
                                                  ProducerProperties.LATEST);
      TimeInterval ti = new TimeInterval(60, Units.MINUTES);
      SecondaryProducer sp = factory.createSecondaryProducer(ti, props);

      TimeInterval historyRP = new TimeInterval(2, Units.HOURS);
      sp.declareTable("userTable", "", historyRP);

      while (true) {
        sp.showSignOfLife();

        try {
          Thread.sleep(55 * 60 * 1000);
        } catch (InterruptedException e) {}
      }
    } catch (RemoteException e) {
      System.err.println("Failed to contact SecondaryProducer service.");
      e.printStackTrace(System.err);
      System.exit(1);
    } catch (UnknownResourceException e) {
      System.err.println("Failed to contact SecondaryProducer resource.");
      e.printStackTrace(System.err);
      System.exit(1);
    } catch (RGMAException e) {
      System.err.println("R-GMA application error in SecondaryProducer.");
      e.printStackTrace(System.err);
      System.exit(1);
    }
  }
}
